﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.Runtime.InteropServices;

namespace FontTest
{
    public partial class Form1 : Form
    {
        private Bitmap imageFont;
        private int fontw = 0;
        private int fonth = 0;
        private int fontgap = 0;




        public Form1()
        {
            InitializeComponent();
            imageFont = null;

        }


        // フォントファイルを読み込んだあとに実行する。
        // 黒以外を黒に、黒を白にする。
        private void setupFontImage()
        {
            if (imageFont == null) return;

            int width = imageFont.Width;
            int height = imageFont.Height;
            uint bmp_pixelVal;

            BitmapData data = imageFont.LockBits(new Rectangle(0, 0, width, height),
                    ImageLockMode.ReadWrite,
                    PixelFormat.Format32bppArgb);
            int stride = data.Stride;

            for (int y = 0; y < height; y++)
            {
                int offset = y * stride;
                for (int x = 0; x < width; x++)
                {
                    bmp_pixelVal = (uint)Marshal.ReadInt32(data.Scan0, offset);
                    if ((bmp_pixelVal&0xFFFFFF) == 0)
                    {
                        bmp_pixelVal = 0xFFFFFFFF;
                    }
                    else
                    {
                        bmp_pixelVal = 0xFF000000;
                    }
                    Marshal.WriteInt32(data.Scan0, offset, (int)bmp_pixelVal);
                    offset += 4;
                }
            }
            imageFont.UnlockBits(data);
        }


        private void ButtonLoadFONT_Click(object sender, EventArgs e)
        {
            bool err = false;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                String s = openFileDialog1.FileName;
                try
                {
                    FileStream fs = new System.IO.FileStream(s,FileMode.Open,FileAccess.Read);
                    imageFont = (Bitmap)Image.FromStream(fs);
                    fs.Close();

                    //imageFont = new Bitmap(s);
                }
                catch
                {
                    imageFont = null;
                    err = true;
                }
                if(!err)
                {
                    int w = imageFont.Width;
                    int h = imageFont.Height;

                    
                    if (w % 16 == 1) fontgap = 1;
                    else fontgap = 0;
                    fontw = (w - fontgap * 17) / 16;
                    fonth = (h - fontgap * 9) / 8;

                    if ((fontw <5)||(fonth <10))
                    {
                        err = true;
                        fontw = 0;
                        fonth = 0;
                        fontgap = 0;
                        imageFont = null;
                    }

                    textBoxFontW.Text = fontw.ToString();
                    textBoxFontH.Text = fonth.ToString();
                    textBoxFontG.Text = fontgap.ToString();
                    setupFontImage();
                }

                if (err)
                {
                    MessageBox.Show("FontLoadError", "FontTest", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }


            }
        }

        private void ButtonView_Click(object sender, EventArgs e)
        {
            draw();
        }



        private void draw()
        {
            int ix = pictureBox1.Size.Width;
            int iy = pictureBox1.Size.Height;

            Graphics g = Graphics.FromImage(pictureBox1.Image);


            g.FillRectangle(Brushes.Black, 0, 0, ix, iy);
            if (imageFont != null)
            {
                string s = textBoxTestText.Text;
                //System.Diagnostics.Debug.WriteLine(s);
                int xpos = 0;
                int ypos = 0;
                int len = s.Length;
                int cpos = 0;

                int drawfontw = fontw;
                int drawfonth = fonth;

                char[] cbuf = new char[4];

                
                while (cpos < len)
                {
                    uint c = s[cpos];

                    if (c == '\r')
                    {
                        xpos = 0;
                    }
                    else if (c == '\n')
                    {
                        ypos += drawfonth;
                    }
                    
                    else
                    {
                        if (c == '^')
                        {
                            
                            cpos++;
                            if (cpos >= len) break;
                            cbuf[0] = s[cpos];
                            if (cbuf[0] == '^')
                            {
                                c = '^';
                            }
                            else
                            {
                                cpos++;
                                if (cpos >= len) break;
                                cbuf[1] = s[cpos];
                                string sx = new string(cbuf,0,2);
                                try
                                {
                                    c = (uint)Convert.ToInt32(sx, 16);
                                }
                                catch
                                {
                                    c = 0x20;
                                }
                            }
                        }

                        //if (c < 0x20 || c > 0x7F)  //範囲外の文字はスペース
                        //{
                        //    c = 0x20;
                        //}

                        int src_col = (int)(c % 0x10);
                        int src_row = (int)(c / 0x10);
                        int src_xpos = (fontgap + fontw) * src_col + fontgap;
                        int src_ypos = (fontgap + fonth) * src_row + fontgap;


                        Rectangle rect = new Rectangle(xpos, ypos, drawfontw, drawfonth);
                        g.DrawImage(imageFont, rect, src_xpos, src_ypos, fontw, fonth, GraphicsUnit.Pixel);


                        xpos += drawfontw;
                    }
 
                    cpos++;
                }

            }
            g.Dispose();
            pictureBox1.Refresh();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Bitmap bmp = new Bitmap(pictureBox1.Size.Width, pictureBox1.Size.Height);
            pictureBox1.Image = bmp;
        }

        private void ButtonCopy_Click(object sender, EventArgs e)
        {
            Clipboard.SetImage(pictureBox1.Image);
        }

        private void ButtonExportFont_Click(object sender, EventArgs e)
        {
            if (imageFont == null) return;

            if (saveFileDialog1.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            String s = saveFileDialog1.FileName;

            using(StreamWriter sw = new StreamWriter(s))
            {
                int xbyte = ((fontw + 7) / 8);
                int buff_size = xbyte * fonth;
                byte[] buff = new byte[buff_size];
                Rectangle rect = new Rectangle(0, 0, fontw, fonth);


                sw.WriteLine(string.Format("unsigned char font[][{0}] = {{", buff_size));

                Bitmap imageChar = new Bitmap(fontw, fonth);
                Graphics g = Graphics.FromImage(imageChar);

                uint c;
                for (c = 0; c <= 0x7F; c++)
                {
                    int x, y, img_offset, buff_offset;
                    int src_col = (int)(c % 0x10);
                    int src_row = (int)(c / 0x10);
                    int src_xpos = (fontgap + fontw) * src_col + fontgap;
                    int src_ypos = (fontgap + fonth) * src_row + fontgap;

                    g.DrawImage(imageFont, rect, src_xpos, src_ypos, fontw, fonth, GraphicsUnit.Pixel);

                    BitmapData data = imageChar.LockBits(new Rectangle(0, 0, fontw, fonth),
                        ImageLockMode.ReadWrite,
                        PixelFormat.Format32bppArgb);
                    int stride = data.Stride;


                    for (int i = 0; i < buff_size; i++)
                    {
                        buff[i] = 0;
                    }

                    for (y = 0; y < fonth; y++)
                    {
                        byte mask = 0x80;
                        buff_offset = y * xbyte;
                        img_offset = y * stride;
                        for (x = 0; x < fontw; x++)
                        {
                            uint pixelVal = (uint)Marshal.ReadInt32(data.Scan0, img_offset);
                            if ((pixelVal & 0xFFFFFF) == 0xFFFFFF)
                            {
                                buff[buff_offset] |= mask;
                            }
                            if (mask == 0x01)
                            {
                                mask = 0x80;
                                buff_offset++;
                            }
                            else
                            {
                                mask >>= 1;
                            }
                            img_offset += 4;
                        }
                    }
                    imageChar.UnlockBits(data);



                    sw.WriteLine(string.Format("    //0x{0:X2}", c));
                    sw.WriteLine("    {");
                    buff_offset = 0;
                    for (y = 0; y < fonth; y++)
                    {
                        sw.Write("     ");
                        for (x = 0; x < xbyte; x++)
                        {
                            sw.Write(string.Format("0x{0:X2}", buff[buff_offset]));
                            buff_offset++;
                            if (buff_offset != buff_size)
                            {
                                sw.Write(",");
                            }

                        }
                        sw.WriteLine("");
                    }
                    sw.Write("    }");
                    if (c < 0x7F)
                    {
                        sw.Write(",");
                    }
                    sw.WriteLine("");

                }
                sw.WriteLine("};");

                g.Dispose();
                imageChar.Dispose();
            }
        }
    }
}
